﻿namespace Connected.Services;

public abstract class ServiceFunction<TArgs, TReturnValue> : ServiceOperation<TArgs>, IFunction<TArgs, TReturnValue>
	where TArgs : IDto
{
	protected TReturnValue Result { get; private set; } = default!;
	public async Task<TReturnValue> Invoke(TArgs args)
	{
		if (args is null)
			throw new ArgumentException(null, nameof(args));

		Arguments = args;

		Result = await OnInvoke();

		return Result;
	}

	protected abstract Task<TReturnValue> OnInvoke();
}
